@@ -10,16 +10,16 @@
 #'
 #' @export
 f_filter <- function(data, ..., .by = NULL){
-  group_info <- tidy_group_info(data, ..., .by = {{ .by }},
-                                ungroup = FALSE,
-                                rename = TRUE)
-  filter_cols <- group_info[["extra_groups"]]
-  filter_df <- cheapr::sset(df_ungroup(group_info[["data"]]), j = filter_cols)
+  group_info <- mutate_summary(data, ..., .by = {{ .by }})
+  filter_cols <- group_info[["new_cols"]]
+  filter_df <- cheapr::sset_col(group_info[["data"]], filter_cols)
   if (df_ncol(filter_df) < 1){
     data
   } else {
-    stopifnot(all(vapply(filter_df, is.logical, FALSE)))
-    df_row_slice(data, cpp_which_all(filter_df))
+    if (!all(vapply(filter_df, is.logical, FALSE))){
+      cli::cli_abort("All expressions in {.fn f_filter} must be logical vectors")
+    }
+    cheapr::sset_df(data, cpp_which_all(filter_df))
   }
 }
 
